package greenfoot.junitUtils;

import greenfoot.junitUtils.runner.TestRunnerState;

/**
 * It remembers the file names of sounds that are played for doing 
 * Test Driven Development (TDD) with JUnit. If we wants to verify 
 * the behaviour of any actor or world for any sound, the test can 
 * verify the played sounds. 
 *
 * @author Francisco Guerra (francisco.guerra@ulpgc.es)
 */
public class SoundPlayed {
	
	/**
	 * It returns the file name of the sound that has played. When there are many
	 * sounds, the file names are returned in the played order.<br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorSound <b>extends</b> Actor {
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             Greenfoot.playSound("anySound.wav");
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the file name of sound that has been played:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.SoundPlayed;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorSoundTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testAnySound() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorSound actor = <b>new</b> ExampleActorSound();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>("anySound.wav", SoundPlayed.get());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @return next sound file name that was played
	 * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 * 
	 */
	public static String get() throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
	    return TestRunnerState.getSoundPlayed();
	}

    /**
     * It tells if it has played a sound associated with a file name. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorSound <b>extends</b> Actor {
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             Greenfoot.playSound("anySound.wav");
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the file name of sound that has been played:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.SoundPlayed;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorSoundTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testAnySound() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorSound actor = <b>new</b> ExampleActorSound();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertTrue</i>(SoundPlayed.get("anySound.wav"));
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
     * @param soundPlayed is the file name of sound that is wanted to verify as played.
     * 
     * @return true when file name is of the sound that has been played.
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     * 
     */
    public static boolean get(String soundPlayed) throws GreenfootRunnerExpected {
    	TestRunnerState.assertRunningUnitTest();
        return TestRunnerState.getSoundPlayed(soundPlayed);
    }

}
