package greenfoot.junitUtils.runner;

import java.util.Random;

import bluej.runtime.ExecServer;
import greenfoot.Actor;
import greenfoot.World;
import greenfoot.core.WorldHandler;
import greenfoot.junitUtils.EventDispatch;
import greenfoot.junitUtils.GreenfootRunnerExpected;
import greenfoot.platforms.standalone.GreenfootUtilDelegateStandAlone;
import greenfoot.util.GreenfootUtil;

/**
 * To initialize data structures for Test Driven Development (TDD) with JUnit
 * 
 * @author Francisco Guerra (francisco.guerra@ulpgc.es)
 */
public class TestRunnerState {
    
    private static boolean      started              = false;
    private static boolean      pause                = false;
	private static World        lastWordStarted      = null;
    private static WorldHandler previousWorldHandler = null;
    
    /**
     * GreenfootRunner initializes junitUtils with every new test
     */
    static synchronized void beginUnitTest() {
    	if (!started) {
    		started              = true;
    		pause                = false;
    		lastWordStarted      = null;
    		previousWorldHandler = WorldHandler.getInstance();
            WorldHandler.initialise();
    	}
        try {
        	GreenfootUtil.getUserName(); 
        } catch (Exception e) {
        	GreenfootUtil.initialise(new GreenfootUtilDelegateStandAlone());
        }
    }

    /**
     * GreenfootRunner initializes junitUtils with every new test
     */
    static synchronized void endUnitTest() {
       	try {
			EventDispatch.keyboardExited();
	    	EventDispatch.mouseExited();
	    	if (started) {
	    		WorldHandler.setInstance(previousWorldHandler);
	    		previousWorldHandler = null;
	    		started              = false;
	    	}

       	} catch (GreenfootRunnerExpected e) {
			e.printStackTrace();
		}
    }
    
    
    /**
     * 
     * @return
     */
    public static boolean runningUnitTest() {
    	return started;
    }
    
    /**
     * 
     * @param world
     */
    public static synchronized void setScenarioWorld(World world) {
    	if (! WorldHandler.getInstance().checkWorldSet()) {
            WorldHandler.getInstance().setWorld(world, false);
    	}
    }
    
    /**
     * 
     * @throws GreenfootRunnerExpected is raised when runningUnitTest is false.
     */
    public static void assertRunningUnitTest() throws GreenfootRunnerExpected {
        if (! TestRunnerState.runningUnitTest()) {
        	throw new GreenfootRunnerExpected();
        }    	
    }

	/**
     * GreenfootRunner clean outputs data structures at beginning of every 
     * new test. Also WorldCreator.getWorld and WorldCreator.runOnce clear 
     * this structures before they are executed
     */
    public static void resetOutputs() {
        pause = false;
        initDelay();
        initSpeed();
        initSoundPlayed();
    }

    /**
     * GreenfootRunner clean outputs data structures at ending of every 
     * test. Also WorldCreator.getWorld and WorldCreator.runOnce clear 
     * this structures after they were executed
     * 
     */
    public static void resetInputs() {
        initRandomNumber();
        initAsk();
        initMicLevel();
    }

    /**
     * 
     * @return true when there is running a test
     */
    public static boolean isStart() {
        return !pause;
    }

    /**
     * Set state as running
     * 
     */
	public static void setStart() {
		pause = false;
	}


    /**
     * 
     * @return true when there is a test that is paused
     */
    public static boolean isStop() {
        return pause;
    }

    /**
     * Set state as paused
     * 
     */
	public static void setStop() {
		lastWordStarted = null;
		pause = true;
	}	

	/**
	 * 
	 * @param world
	 */
	public static void setStarted(World world) {
    	if (world != null && lastWordStarted != world) {
    		world.started();
    		lastWordStarted = world;
    	}
	}	

    /**
     * When there is any stored integer, the operation 
     * getRandomNumber return this integer in FIFO order.
     * 
     */
    private static java.util.HashMap<String, java.util.ArrayList<Integer>> randomQueue
        = new java.util.HashMap<String, java.util.ArrayList<Integer>>();

    /**
     * 
     * To initialize the random numbers queue 
     * 
     */
    private static void initRandomNumber() {
        randomQueue.clear();
    }

    /**
     * 
     * @param nextNumber will be the value returned by the operation
     *                   getRandomNumber in FIFO order.
     * @param callerClass is the class of object that can receive nextNumber.
     *                    This class must derived from Actor or World.
     */
    public static void setRandom(int nextNumber, Class<?> callerClass) {
        String callerClassName;
        if (Actor.class.isAssignableFrom(callerClass)){
            callerClassName = callerClass.getName();
            
        } else if (World.class.isAssignableFrom(callerClass)){
            callerClassName = callerClass.getName();
            
        } else {
            callerClassName = Object.class.getName();
        }
        
        if (!randomQueue.containsKey(callerClassName)) {
            randomQueue.put(callerClassName, new java.util.ArrayList<Integer>());
        }
        randomQueue.get(callerClassName).add(nextNumber);
    }
    
    /**
     * The random numbers can be associated with a actor or a world
     * 
     * @param randomGenerator is Greenfoot.randomGenerator
     * @param limit is the value received by Greenfoot.getRandomNumber
     * 
     */
    public static int getRandomNumber(Random randomGenerator, int limit) {
        final String objectClassName = Object.class.getName();
        String callerClassName = objectClassName;
        for (StackTraceElement traceElement: Thread.currentThread().getStackTrace()) {
            try {
                if (Actor.class.isAssignableFrom(getClass(traceElement.getClassName()))){
                    callerClassName = traceElement.getClassName();
                    break;
                    
                } else if (World.class.isAssignableFrom(getClass(traceElement.getClassName()))){
                    callerClassName = traceElement.getClassName();
                    break;
                }
            } catch (ClassNotFoundException e) {
            }
        }
        
        if (randomQueue.containsKey(callerClassName)) {
            if (randomQueue.get(callerClassName).isEmpty()){
                return randomGenerator.nextInt(limit);
            
            } else {
                return randomQueue.get(callerClassName).remove(0);
            }
            
        } else if (randomQueue.containsKey(objectClassName)
                   && !randomQueue.get(objectClassName).isEmpty()){
            return randomQueue.get(objectClassName).remove(0);
            
        } else {
            return randomGenerator.nextInt(limit);
        } 
    }
    
    /**
     * When there is any stored String, the operation 
     * getAsk return this string in FIFO order.
     * 
     * The ask can be associated with a actor or a world
     * 
     */
    private static java.util.HashMap<String, java.util.ArrayList<String>> askQueue
        = new java.util.HashMap<String, java.util.ArrayList<String>>();

    /**
     * To initialize the ask queue 
     *
     */
    private static void initAsk() {
        askQueue.clear();
    }

    /**
     * To extract the next ask in FIFO order
     * 
     * The ask can be associated with a actor or a world
     * 
     * @return the oldest ask
     */
    public static String getAsk() {
        final String objectClassName = Object.class.getName();
        String callerClassName = objectClassName;
        for (StackTraceElement traceElement: Thread.currentThread().getStackTrace()) {
            try {
                if (Actor.class.isAssignableFrom(getClass(traceElement.getClassName()))){
                    callerClassName = traceElement.getClassName();
                    break;
                    
                } else if (World.class.isAssignableFrom(getClass(traceElement.getClassName()))){
                    callerClassName = traceElement.getClassName();
                    break;
                }
            } catch (ClassNotFoundException e) {
            }
        }
        
        if (askQueue.containsKey(callerClassName)) {
            if (askQueue.get(callerClassName).isEmpty()){
                return "";
            
            } else {
                return askQueue.get(callerClassName).remove(0);
            }
            
        } else if (askQueue.containsKey(objectClassName)
                   && !askQueue.get(objectClassName).isEmpty()){
            return askQueue.get(objectClassName).remove(0);
            
        } else {
            return "";
        } 
    }
    
    /**
     * 
     * @param nextNumber will be the value returned by the operation
     *                   getRandomNumber in FIFO order.
     * @param callerClass is the class of object that can receive nextNumber.
     *                    This class must derived from Actor or World.
     */
    public static void setAsk(String nextAsk, Class<?> callerClass) {
        String callerClassName;
        if (Actor.class.isAssignableFrom(callerClass)){
            callerClassName = callerClass.getName();
            
        } else if (World.class.isAssignableFrom(callerClass)){
            callerClassName = callerClass.getName();
            
        } else {
            callerClassName = Object.class.getName();
        }
        
        if (!askQueue.containsKey(callerClassName)) {
            askQueue.put(callerClassName, new java.util.ArrayList<String>());
        }
        askQueue.get(callerClassName).add(nextAsk);
    } 

    /**
     * When there is any stored integer, the operation 
     * Greenfoot.getMicLevel return this integer in FIFO order.
     * 
     */
    private static java.util.ArrayList<Integer> micLevelQueue = new java.util.ArrayList<Integer>();

    /**
     * 
     * To initialize the mic levels queue
     * 
     */
    private static void initMicLevel() {
        micLevelQueue.clear();
    }

    /**
     * 
     * @param level is next level of mic for testing
     */
    public static void setMicLevel(int level)
    {
        if (started){
        	micLevelQueue.add(level);
        }
    }
    
	public static int getMicLevel() {
    	if (micLevelQueue.isEmpty()) {
    		return -1;
    		
    	} else {
    		return micLevelQueue.remove(0);
    	}
	}

    /**
     * When there is any stored integer, the operation 
     * getDelay return this integer in FIFO order.
     * 
     */
    private static java.util.ArrayList<Integer> delayQueue = new java.util.ArrayList<Integer>();

    /**
     * 
     * To initialize the delay queue
     * 
     */
    private static void initDelay() {
        delayQueue.clear();
    }


    /**
     * 
     * @return delay was done
     */
    public static int getDelay() {
        if (delayQueue.isEmpty()) {
            return -1;
            
        } else {
            return delayQueue.remove(0);
        }
    }
    
	public static void setDelay(int time) {
		delayQueue.add(time);		
	}

	/**
     * 
     * @param delay
     * @return false when delay was not done
     */
    public static boolean getDelay(int delay) {
        return delayQueue.remove((Integer)delay);
    }
    
    /**
     * When there is any stored integer, the operation 
     * getDelay return this integer in FIFO order.
     * 
     */
    private static java.util.ArrayList<Integer> speedQueue = new java.util.ArrayList<Integer>();

    /**
     * To initialize the speed queue
     * 
     */
    private static void initSpeed() {
        speedQueue.clear();
    }

    /**
     * 
     * @return speed was done
     */
    public static int getSpeed() {
        if (speedQueue.isEmpty()) {
            return -1;
            
        } else {
            return speedQueue.remove(0);
        }
    }
    
    /**
     * 
     * @param speed
     */
	public static void setSpeed(int speed) {
		speedQueue.add(speed);
	}

	/**
     * 
     * @param speed
     * @return false when this speed was not done
     */
    public static boolean getSpeed(int speed) {
        return speedQueue.remove((Integer)speed);
    }
    

    private static java.util.ArrayList<String> soundPlayedQueue = new java.util.ArrayList<String>();

    /**
     * To initialize the sound queue
     * 
     */
    private static void initSoundPlayed() {
        soundPlayedQueue.clear();
    }
    
    /**
     * 
     * @return sound file name was played
     */
    public static String getSoundPlayed() {
        if (soundPlayedQueue.isEmpty()) {
            return "";
            
        } else {
            return soundPlayedQueue.remove(0);
        }
    }

	public static void setSound(String soundFile) {
		soundPlayedQueue.add(soundFile);
	}

	/**
     * 
     * @param soundPlayed
     * 
     * @return false when sound file name was not played
     */
    public static boolean getSoundPlayed(String soundPlayed) {
        return soundPlayedQueue.remove(soundPlayed);
    }
    

    private static Class<?> getClass(String className) throws ClassNotFoundException{
    	try {
    		return Class.forName(className);
    	} catch (Exception e) {
  	    	return ExecServer.loadAndInitClass(className);
      	}
//    	Class<?> cls = ExecServer.loadAndInitClass(className);
//    	if (cls == null) {
//    		return Class.forName(className);
//    	} else {
//    		return ExecServer.loadAndInitClass(className);
//    	}
    }

}
